#ifndef DISPLAY_H
#define DISPLAY_H

#include <system.h>

#include "display_font.h"

#define DIRECTION_OUTPUT 0x00
#define DIRECTION_INPUT 0xff

#define LCD_SET_COL_HI 0x10
#define LCD_SET_COL_LO 0x00
#define LCD_SET_PAGE 0xb0
#define LCD_SET_ADC_NOR 0xa0
#define LCD_SET_ADC_REV 0xa1
#define LCD_SET_SHL_NOR 0xc0
#define LCD_SET_SHL_REV 0xc8
#define LCD_REG_RESISTOR 0x20
#define LCD_REF_VOLT_MODE 0x81
#define LCD_REF_VOLT_REG 0x00
#define LCD_PWR_CTL 0x28
#define LCD_RESET 0xe2
#define LCD_SET_BIAS_0 0xa2
#define LCD_SET_BIAS_1 0xa3
#define LCD_DISP_ON 0xaf

#define DISPLAY_COLUMN_COUNT 128
#define DISPLAY_PAGE_COUNT 8
#define DISPLAY_COLUMN_OFFSET 0

#define DISPLAY_E porta.4
#define DISPLAY_RW porta.3
#define DISPLAY_RS porta.2
#define DISPLAY_RST porta.1
#define DISPLAY_CS porta.0

#define DISPLAY_DATA portd
#define DISPLAY_DATA_DDR trisd

#define DISPLAY_STATUS_BUSY 0x80

enum display_constants {
	DISPLAY_WIDTH = 128,
	DISPLAY_HEIGHT = 64
};

typedef enum display_fonts
{
	DISPLAY_FONT_STANDARD,
	DISPLAY_FONT_LARGE
} display_fonts_t;

void display_initialize (void);
void display_write_command (unsigned char command);
void display_write_data (unsigned char data);
unsigned char display_read_status (void);
unsigned char display_read_data (void);
void display_clear (void);
void display_bitmap (const unsigned char *data0, const unsigned char *data1,
	const unsigned char *data2, const unsigned char *data3);
void display_goto (unsigned char column, unsigned char page);
void display_set_pixel (unsigned char x, unsigned char y, unsigned char color);

void display_draw_line (unsigned char x1, unsigned char y2,
	unsigned char x2, unsigned char y2);

void display_draw_string (unsigned char x, unsigned char y,
	display_fonts_t font, const char *string);

void display_draw_character (unsigned char x, unsigned char y,
	display_fonts_t font, char character);

#endif